# -*- coding: utf-8 -*-
from pathlib import Path

import click

from ros_d2.export import export


@click.group()
def cli():
    pass


@cli.command("export", help="Export ROS 2 node graph to D2")
@click.argument("output", type=str)
def export_command(output: str):
    output_file = Path(output)

    # If the output_file is not a .d2 file, add the extension.
    if output_file.suffix != ".d2":
        output_file = output_file.with_suffix(".d2")

    export(output_file)


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from pathlib import Path

from ros_d2.helpers.convert import convert
from ros_d2.helpers.node_info import get_node_info


def export(output_file: Path):
    node_infos = get_node_info()
    d2_graph = convert(node_infos)
    open(output_file, "w+").write(d2_graph)



# -*- coding: utf-8 -*-
from typing import List

from py_d2.D2Connection import D2Connection, Direction
from py_d2.D2Diagram import D2Diagram
from py_d2.D2Shape import D2Shape, Shape
from py_d2.D2Style import D2Style

from ros_d2.helpers.node_info import NodeInfo

node_style = D2Style(
    opacity=0.6,
    three_d=True,
)


def convert(node_infos: List[NodeInfo]) -> str:
    """Converts a list of NodeInfo objects to a d2 graph."""
    # Get all the unique topics
    topics_unique = set()
    for node_info in node_infos:
        for sub in node_info.subs:
            topics_unique.add(sub.topic_name)
        for pub in node_info.pubs:
            topics_unique.add(pub.topic_name)

    # Convert to D2
    diagram = D2Diagram()
    for topic in topics_unique:
        shape = D2Shape(name=topic, shape=Shape.classs)
        diagram.add_shape(shape)

    for node_info in node_infos:
        shape = D2Shape(name=node_info.node_name, style=node_style)
        diagram.add_shape(shape)
        for sub in node_info.subs:
            diagram.add_connection(
                D2Connection(
                    shape_1=node_info.node_name,
                    shape_2=sub.topic_name,
                    direction=Direction.TO,
                )
            )
        for pub in node_info.pubs:
            diagram.add_connection(
                D2Connection(
                    shape_1=node_info.node_name,
                    shape_2=pub.topic_name,
                    direction=Direction.FROM,
                )
            )

    return str(diagram)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Tuple

from ros2cli.node.strategy import NodeStrategy
from ros2node.api import get_node_names

TOPICS_TO_IGNORE = [
    "/parameter_events",
    "/rosout",
]


@dataclass
class TopicInfo:
    topic_name: str
    topic_types: List[str]


@dataclass
class NodeInfo:
    node_name: str
    subs: List[TopicInfo]
    pubs: List[TopicInfo]


def filter_topics(topics: List[TopicInfo]) -> List[TopicInfo]:
    return [t for t in topics if t.topic_name not in TOPICS_TO_IGNORE]


def topic_tuple_to_info(topic_tuple: Tuple[str, List[str]]) -> TopicInfo:
    return TopicInfo(topic_name=topic_tuple[0], topic_types=topic_tuple[1])


def get_node_info() -> List[NodeInfo]:
    node_infos: List[NodeInfo] = []
    with NodeStrategy([]) as node:
        node_names = get_node_names(node=node)
        for n in node_names:
            node_name = n.full_name
            subs_tuple = node.get_subscriber_names_and_types_by_node(
                n.name, n.namespace
            )
            pubs_tuple = node.get_publisher_names_and_types_by_node(n.name, n.namespace)
            subs = [topic_tuple_to_info(t) for t in subs_tuple]
            pubs = [topic_tuple_to_info(t) for t in pubs_tuple]

            subs = filter_topics(subs)
            pubs = filter_topics(pubs)

            node_info = NodeInfo(node_name=node_name, subs=subs, pubs=pubs)
            node_infos.append(node_info)

    return node_infos
